#include <Arduino.h>
#include "utils.h"
#include "readers.h"
#include "vars.h"

volatile int PN532Reader::irq = 0;

PN532Reader::PN532Reader(uint8_t clk, uint8_t miso, uint8_t mosi, uint8_t ss) {
	pn532 = new Adafruit_PN532(clk, miso, mosi, ss);
}

void PN532Reader::begin() {
	pn532->begin();
	irq = 0;
	pn532->startPassiveTargetIDDetection(PN532_MIFARE_ISO14443A);
}

void PN532Reader::end() {
	pn532->sendCommandCheckAck(new byte[1]{PN532_COMMAND_POWERDOWN}, 1, 1);
}

bool PN532Reader::isNewCardPresent() {
	return irq > 1;
}

String PN532Reader::getCardUid() {
	pn532->readDetectedPassiveTargetID(uid, &uidLength);
	Serial.println(byteArrayAsHexString(uid, uidLength));
	return byteArrayAsHexString(uid, uidLength);
}

void PN532Reader::reset() {
	this->end();
	this->begin();
}

inline bool PN532Reader::canHaveUnstableIdentifier() {
	return true;
}

void PN532Reader::init() {
	pn532->begin();
	uint32_t version = pn532->getFirmwareVersion();
	if (!version) {
		Serial.print("Couldn't find PN53x board");
		return;
		//esp_restart();
	}

	Serial.print("Found chip PN5"); Serial.println((version>>24) & 0xFF, HEX);
	Serial.print("Firmware ver. "); Serial.print((version>>16) & 0xFF, DEC);
	Serial.print('.'); Serial.println((version>>8) & 0xFF, DEC);
}
